/* Copyright (c) 2024 Skyward Experimental Rocketry
 * Author: Federico Lolli
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

// #include <filesystem/console/console_device.h>
#include <mavlink_lib/lyra/mavlink.h>
// SX1278 includes
#include <radio/MavlinkDriver/MavlinkDriver.h>
#include <radio/SerialTransceiver/SerialTransceiver.h>
#include <shared/radio/Radio.h>

#include <iostream>
#include <thread>

using namespace miosix;
using namespace Boardcore;
using namespace Arpist;

// SerialTransceiver transceiver(usart);
// MavlinkDriver<20, 10> mavlink(&transceiver);

// === CONSTANTS ===
/** @brief End of transmission character */
constexpr uint8_t ACK = 0x06;

// === DEFINITIONS ===
mavlink_message_t readPacketFromSerial();

// === MAIN ===
int main()
{
    while (true)
    {
        mavlink_payload_flight_tm_t payload;

        mavlink_message_t msg = readPacketFromSerial();
        printf("Received message with ID %d\n", msg.msgid);
        mavlink_msg_payload_flight_tm_decode(&msg, &payload);
        printf("Received payload with timestamp %lld\n", payload.timestamp);
        std::this_thread::sleep_for(std::chrono::milliseconds(100));
    }

    return 0;
}

/**
 * @brief Read a packet from the serial port
 * @warning This function will parse raw bytes coming from
 * serial into the struct
 * @return mavlink_payload_flight_tm_t
 */
mavlink_message_t readPacketFromSerial()
{
    bool stop_flag = false;
    ssize_t rcv_size;
    uint8_t serial_buffer[128];
    uint8_t parse_result = 0;
    mavlink_message_t msg;
    mavlink_status_t status;

    auto serial = DefaultConsole::instance().get();

    while (!stop_flag)
    {
        // Check for a new message on the device
        rcv_size = serial->readBlock(serial_buffer, 128, 0);
        // printf("Received %d bytes\n", rcv_size);

        // If there's a new message ...
        if (rcv_size > 0)
        {
            for (int i = 0; i < rcv_size; i++)
            {
                parse_result = 0;
                // ... parse received bytes
                parse_result =
                    mavlink_parse_char(MAVLINK_COMM_0,
                                       serial_buffer[i],  // byte to parse
                                       &msg,              // where to parse it
                                       &status);          // stats to update

                // When a valid message is found ...
                if (parse_result == 1)
                {
                    stop_flag = true;
                    break;
                }
            }
        }
    }

    serial->writeBlock(&ACK, 1, 0);

    return msg;
}
