/* Copyright (c) 2024 Skyward Experimental Rocketry
 * Author: Federico Lolli
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#pragma once

#include <Singleton.h>
#include <config/Mavlink.h>
#include <config/RadioConfig.h>
#include <radio/MavlinkDriver/MavlinkDriver.h>
#include <radio/SX1278/SX1278Fsk.h>

#if defined _BOARD_STM32F429ZI_NOKIA
#include "interfaces-impl/hwmapping.h"

#define SX1278_SPI SPI4
#define SX1278_IRQ_DIO0 EXTI6_IRQHandlerImpl
#define SX1278_IRQ_DIO1 EXTI4_IRQHandlerImpl
#define SX1278_IRQ_DIO3 EXTI11_IRQHandlerImpl

#else
#error "Target not supported"
#endif

namespace Arpist
{

#if defined _BOARD_STM32F429ZI_NOKIA
// pin mapping
using radio_cs   = miosix::peripherals::ra01::pc13::cs;
using radio_dio0 = miosix::peripherals::ra01::pc13::dio0;
using radio_dio1 = miosix::peripherals::ra01::pc13::dio1;
using radio_dio3 = miosix::peripherals::ra01::pc13::dio3;
using radio_sck  = miosix::interfaces::spi4::sck;
using radio_miso = miosix::interfaces::spi4::miso;
using radio_mosi = miosix::interfaces::spi4::mosi;
#endif

using RadioMavDriver =
    Boardcore::MavlinkDriver<Boardcore::SX1278Fsk::MTU,
                             Arpist::MAV_OUT_QUEUE_SIZE,
                             MAVLINK_MAX_DIALECT_PAYLOAD_SIZE>;

// static volatile int dio0_cnt = 0;
// static volatile int dio1_cnt = 0;
// static volatile int dio3_cnt = 0;

/**
 * @brief Radio class
 */
class Radio : public Boardcore::Singleton<Radio>, public Boardcore::Transceiver
{
    friend class Boardcore::Singleton<Radio>;

public:
    void handleDioIRQ();

    bool start(std::function<void(const mavlink_message_t&)> onReceive);

    bool sendMsg(const mavlink_message_t& msg);

private:
    ssize_t receive(uint8_t* pkt, size_t max_len) override;

    bool send(uint8_t* pkt, size_t len) override;

    bool started = false;
    Boardcore::SPIBus sx1278_bus{SX1278_SPI};
    std::unique_ptr<Boardcore::SX1278Fsk> sx1278;
    std::unique_ptr<RadioMavDriver> mav_driver;
};

}  // namespace Arpist
