use std::path::PathBuf;

use clap::{
    builder::{
        styling::{AnsiColor, Color, Style},
        Styles,
    },
    Parser,
};

#[derive(Debug, Parser)]
#[command(styles=get_styles())]
pub struct Cli {
    /// The UDP port to use
    #[clap(short, long, value_name = "PORT", default_value = "42069")]
    pub port: u16,

    /// Origin of the messages (System ID)
    #[clap(short, long, default_value = "171")]
    pub system_id: u8,

    /// The high rate telemetry CSV file to read from
    #[clap(value_name = "HIGH_RATE_MAIN_FILE")]
    pub high_rate_file_main: PathBuf,

    /// The high rate telemetry CSV file to read from
    #[clap(value_name = "HIGH_RATE_PAYLOAD_FILE")]
    pub high_rate_file_payload: PathBuf,

    /// The log level
    #[clap(short, long)]
    pub verbose: bool,
}

pub fn get_styles() -> Styles {
    Styles::styled()
        .usage(
            Style::new()
                .bold()
                .underline()
                .fg_color(Some(Color::Ansi(AnsiColor::Yellow))),
        )
        .header(
            Style::new()
                .bold()
                .underline()
                .fg_color(Some(Color::Ansi(AnsiColor::Yellow))),
        )
        .literal(Style::new().fg_color(Some(Color::Ansi(AnsiColor::Green))))
        .invalid(
            Style::new()
                .bold()
                .fg_color(Some(Color::Ansi(AnsiColor::Red))),
        )
        .error(
            Style::new()
                .bold()
                .fg_color(Some(Color::Ansi(AnsiColor::Red))),
        )
        .valid(
            Style::new()
                .bold()
                .underline()
                .fg_color(Some(Color::Ansi(AnsiColor::Green))),
        )
        .placeholder(Style::new().fg_color(Some(Color::Ansi(AnsiColor::White))))
}
